/*****************************************************************************
 * Joseph Zambreno
 * Department of Electrical and Computer Engineering
 * Iowa State University
 *****************************************************************************/

/*****************************************************************************
 * bmp_utils.h - utilities for reading .bmp files. 
 *
 *
 * NOTES:
 * 4/15/11 by JAZ::Design created.
 *****************************************************************************/


#ifndef _BMP_UTILS_H_
#define _BMP_UTILS_H_

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "boomtown.h"

// BMP header data structures

// The 2 byte magic number, as a separate struct to avoid
// alignment problems
typedef struct {
  uint8_t magic[2];
} bmp_magic;

// File size and offset information
typedef struct {
  uint32_t filesz;
  uint16_t creator1;
  uint16_t creator2;
  uint32_t bmp_offset;
} bmp_header;
 
// Bitmap information (assumes Windows V3 version)
typedef struct {
  uint32_t header_sz;
  uint32_t width;
  uint32_t height;
  uint16_t nplanes;
  uint16_t bitspp;
  uint32_t compress_type;
  uint32_t bmp_bytesz;
  uint32_t hres;
  uint32_t vres;
  uint32_t ncolors;
  uint32_t nimpcolors;
} dib_header;

// Struct to hold all the metadata, including the color palette
typedef struct {
  bmp_magic *h1;
  bmp_header *h2;
  dib_header *h3;
  uint8_t *h4;
} bmp_file_info;


uint8_t *bmpRead(uint8_t *bmp_fname, uint32_t *width, uint32_t *height);
void bmpInfo(bmp_file_info *bmp);

#endif
