/*****************************************************************************
 * Joseph Zambreno               
 * Department of Electrical and Computer Engineering
 * Iowa State University
 *****************************************************************************/

/*****************************************************************************
 * gl_utils.c - OpenGL utility functions for the boomtown application.
 *
 *
 * NOTES:
 * 4/15/11 by JAZ::Design created.
 *****************************************************************************/

#include "boomtown.h"
#include "bmp_utils.h"


/*****************************************************************************
 * Function: drawScoreboard                           
 * Description: Draws the scoreboard, based on each team's core and player
 * status
 *****************************************************************************/
void drawScoreboard() {

  float TC_x1, TC_x2;
  int8_t i;
  uint8_t len;


  // Let's flush the scoreboard for purposes of simplicity
  for (i = 0; i < 4; i++) {
    if (game.score[i] < -9)
      game.score[i] = -9;

    if (game.score[i] > 19)
      game.score[i] = 19;
  }


  glBindTexture(GL_TEXTURE_2D, textures[7]);

  // Note that the scoreboard does not have to worry about depth
  glDisable(GL_DEPTH_TEST);

  glBegin(GL_QUADS);

  // Draw the two edges of the scoreboard as a simple mapped quad
  glTexCoord2d(192.0/208, 0.0);glVertex2f(-5.0, game.grid_height+1);
  glTexCoord2d(1.0, 0.0);glVertex2f(-1.0, game.grid_height+1);
  glTexCoord2d(1.0, 16.0/80.0);glVertex2f(-1.0, -1.0);
  glTexCoord2d(192.0/208, 16.0/80.0);glVertex2f(-5.0, -1.0);

  glTexCoord2d(192.0/208, 0.0);glVertex2f(game.grid_width+1, 
					  game.grid_height+1);
  glTexCoord2d(1.0, 0.0);glVertex2f(game.grid_width+6, game.grid_height+1);
  glTexCoord2d(1.0, 16.0/80.0);glVertex2f(game.grid_width+6, -1.0);
  glTexCoord2d(192.0/208, 16.0/80.0);glVertex2f(game.grid_width+1, -1.0);  


  // Team 1 score and player status
  if (game.score[0] < 0) {
    TC_x1 = 192.0/208;
    TC_x2 = -1.0*game.score[0]*16/208;
  }
  else if (game.score[0] == 0) {
    TC_x1 = 160.0/208;
    TC_x2 = 0.0;
  }
  else if (game.score[0] > 9) {
    TC_x1 = 176.0/208;
    TC_x2 = (game.score[0] % 10)*16.0/208;
  }
  else {
    TC_x1 = 160.0/208;
    TC_x2 = game.score[0]*16.0/208;
  }

  glTexCoord2d(TC_x1, 65.0/80);glVertex2f(-4.0, 1.0);
  glTexCoord2d(TC_x1+15.0/208, 65.0/80);glVertex2f(-3.0, 1.0);
  glTexCoord2d(TC_x1+15.0/208, 1.0);glVertex2f(-3.0, 0.0);
  glTexCoord2d(TC_x1, 1.0);glVertex2f(-4.0, 0.0);

  glTexCoord2d(TC_x2, 65.0/80);glVertex2f(-3.0, 1.0);
  glTexCoord2d(TC_x2+15.0/208, 65.0/80);glVertex2f(-2.0, 1.0);
  glTexCoord2d(TC_x2+15.0/208, 1.0);glVertex2f(-2.0, 0.0);
  glTexCoord2d(TC_x2, 1.0);glVertex2f(-3.0, 0.0);

  for (i = 0; i < 4; i++) {
    TC_x1 = game.teams[0].players[i].status * 16.0/208;
    glTexCoord2d(TC_x1, 49.0/80);glVertex2f(-4.0+i%2, 2.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 49.0/80);glVertex2f(-3.0+i%2, 2.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 64.0/80);glVertex2f(-3.0+i%2, 1.0+i/2);
    glTexCoord2d(TC_x1, 64.0/80);glVertex2f(-4.0+i%2, 1.0+i/2);    
  }



  // Team 2 score and player status
  if (game.score[1] < 0) {
    TC_x1 = 192.0/208;
    TC_x2 = -1.0*game.score[1]*16/208;
  }
  else if (game.score[1] == 0) {
    TC_x1 = 160.0/208;
    TC_x2 = 0.0;
  }
  else if (game.score[1] > 9) {
    TC_x1 = 176.0/208;
    TC_x2 = (game.score[1] % 10)*16.0/208;
  }
  else {
    TC_x1 = 160.0/208;
    TC_x2 = game.score[1]*16.0/208;
  }

  glTexCoord2d(TC_x1, 65.0/80);glVertex2f(-4.0, game.grid_height-2);
  glTexCoord2d(TC_x1+15.0/208, 65.0/80);glVertex2f(-3.0, game.grid_height-2);
  glTexCoord2d(TC_x1+15.0/208, 1.0);glVertex2f(-3.0, game.grid_height-3);
  glTexCoord2d(TC_x1, 1.0);glVertex2f(-4.0, game.grid_height-3);

  glTexCoord2d(TC_x2, 65.0/80);glVertex2f(-3.0, game.grid_height-2);
  glTexCoord2d(TC_x2+15.0/208, 65.0/80);glVertex2f(-2.0, game.grid_height-2);
  glTexCoord2d(TC_x2+15.0/208, 1.0);glVertex2f(-2.0, game.grid_height-3);
  glTexCoord2d(TC_x2, 1.0);glVertex2f(-3.0, game.grid_height-3);

  for (i = 0; i < 4; i++) {
    TC_x1 = game.teams[1].players[i].status * 16.0/208;
    glTexCoord2d(TC_x1, 33.0/80);glVertex2f(-4.0+i%2, 
					    game.grid_height-1.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 33.0/80);glVertex2f(-3.0+i%2, 
					    game.grid_height-1.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 48.0/80);glVertex2f(-3.0+i%2, 
					    game.grid_height-2.0+i/2);
    glTexCoord2d(TC_x1, 48.0/80);glVertex2f(-4.0+i%2, 
					    game.grid_height-2.0+i/2);
  }


  // Team 3 score and player status
  if (game.score[2] < 0) {
    TC_x1 = 192.0/208;
    TC_x2 = -1.0*game.score[2]*16/208;
  }
  else if (game.score[2] == 0) {
    TC_x1 = 160.0/208;
    TC_x2 = 0.0;
  }
  else if (game.score[2] > 9) {
    TC_x1 = 176.0/208;
    TC_x2 = (game.score[2] % 10)*16.0/208;
  }
  else {
    TC_x1 = 160.0/208;
    TC_x2 = game.score[2]*16.0/208;
  }

  glTexCoord2d(TC_x1, 65.0/80);glVertex2f(game.grid_width+2.0, 1.0);
  glTexCoord2d(TC_x1+15.0/208, 65.0/80);glVertex2f(game.grid_width+3.0, 1.0);
  glTexCoord2d(TC_x1+15.0/208, 1.0);glVertex2f(game.grid_width+3.0, 0.0);
  glTexCoord2d(TC_x1, 1.0);glVertex2f(game.grid_width+2.0, 0.0);

  glTexCoord2d(TC_x2, 65.0/80);glVertex2f(game.grid_width+3.0, 1.0);
  glTexCoord2d(TC_x2+15.0/208, 65.0/80);glVertex2f(game.grid_width+4.0, 1.0);
  glTexCoord2d(TC_x2+15.0/208, 1.0);glVertex2f(game.grid_width+4.0, 0.0);
  glTexCoord2d(TC_x2, 1.0);glVertex2f(game.grid_width+3.0, 0.0);

  for (i = 0; i < 4; i++) {
    TC_x1 = game.teams[2].players[i].status * 16.0/208;
    glTexCoord2d(TC_x1, 17.0/80);glVertex2f(game.grid_width+2.0+i%2, 
					    2.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 17.0/80);glVertex2f(game.grid_width+3.0+i%2, 
					    2.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 32.0/80);glVertex2f(game.grid_width+3.0+i%2, 
					    1.0+i/2);
    glTexCoord2d(TC_x1, 32.0/80);glVertex2f(game.grid_width+2.0+i%2, 
					    1.0+i/2);    
  }



  // Team 4 score and player status
  if (game.score[3] < 0) {
    TC_x1 = 192.0/208;
    TC_x2 = -1.0*game.score[3]*16/208;
  }
  else if (game.score[3] == 0) {
    TC_x1 = 160.0/208;
    TC_x2 = 0.0;
  }
  else if (game.score[3] > 9) {
    TC_x1 = 176.0/208;
    TC_x2 = (game.score[3] % 10)*16.0/208;
  }
  else {
    TC_x1 = 160.0/208;
    TC_x2 = game.score[3]*16.0/208;
  }

  glTexCoord2d(TC_x1, 65.0/80);glVertex2f(game.grid_width+2.0, 
					  game.grid_height-2);
  glTexCoord2d(TC_x1+15.0/208, 65.0/80);glVertex2f(game.grid_width+3.0, 
						   game.grid_height-2);
  glTexCoord2d(TC_x1+15.0/208, 1.0);glVertex2f(game.grid_width+3.0, 
					       game.grid_height-3);
  glTexCoord2d(TC_x1, 1.0);glVertex2f(game.grid_width+2.0,
				      game.grid_height-3);

  glTexCoord2d(TC_x2, 65.0/80);glVertex2f(game.grid_width+3.0, 
					  game.grid_height-2);
  glTexCoord2d(TC_x2+15.0/208, 65.0/80);glVertex2f(game.grid_width+4.0, 
						   game.grid_height-2);
  glTexCoord2d(TC_x2+15.0/208, 1.0);glVertex2f(game.grid_width+4.0, 
					       game.grid_height-3);
  glTexCoord2d(TC_x2, 1.0);glVertex2f(game.grid_width+3.0, 
				      game.grid_height-3);

  for (i = 0; i < 4; i++) {
    TC_x1 = game.teams[3].players[i].status * 16.0/208;
    glTexCoord2d(TC_x1, 0.0/80);glVertex2f(game.grid_width+2.0+i%2, 
					    game.grid_height-1.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 0.0/80);glVertex2f(game.grid_width+3.0+i%2, 
					    game.grid_height-1.0+i/2);
    glTexCoord2d(TC_x1+15.0/208, 16.0/80);glVertex2f(game.grid_width+3.0+i%2, 
					    game.grid_height-2.0+i/2);
    glTexCoord2d(TC_x1, 16.0/80);glVertex2f(game.grid_width+2.0+i%2, 
					    game.grid_height-2.0+i/2);
  }



 
  glEnd();
  glFlush();


  // Draw each team's name in the appropriate place
  glBindTexture(GL_TEXTURE_2D, textures[8]);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  len = strlen(game.teams[0].name);
  if (len > 8) {
    len = 8;
  }
  glColor3ub(220, 220, 220);
  drawText(game.teams[0].name, len, -3.0, 3.5, 0.5, 1.0);

  len = strlen(game.teams[1].name);
  if (len > 8) {
    len = 8;
  }
  glColor3ub(85, 85, 85);
  drawText(game.teams[1].name, len, -3.0, game.grid_height-3.5, 0.5, 1.0);

  len = strlen(game.teams[2].name);
  if (len > 8) {
    len = 8;
  }
  glColor3ub(0, 0, 184);
  drawText(game.teams[2].name, len, game.grid_width+3.0, 3.5, 0.5, 1.0);

  len = strlen(game.teams[3].name);
  if (len > 8) {
    len = 8;
  }
  glColor3ub(224, 0, 0);
  drawText(game.teams[3].name, len, game.grid_width+3.0, 
	   game.grid_height-3.5, 0.5, 1.0);


  glColor3ub(255, 255, 255);
  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);

}


/*****************************************************************************
 * Function: drawText
 * Description: Draw a string or length len at the specified position
 *****************************************************************************/
void drawText(uint8_t *str, uint8_t len, float x_cen, float y_cen, 
	      float width, float height) {

  uint8_t i;
  int8_t tchar;
  float x;
  float y;

  y = y_cen - height/2;
  x = x_cen - width/2*len;


  // Draw each team's name in the appropriate place
  glBegin(GL_QUADS);

  for (i = 0; i < len; i++) {

    tchar = str[i];
    if (tchar >= 'a') {
      tchar = tchar - 'a';
    }
    else {
      tchar = tchar - 'A';
    }
    if ((tchar >= 0) && (tchar <= 25)) {
      glTexCoord2d(tchar/26.0, 0.0);glVertex2f(x, y+height);
      glTexCoord2d((tchar+1)/26.0, 0.0);glVertex2f(x+width, y+height);
      glTexCoord2d((tchar+1)/26.0, 1.0);glVertex2f(x+width, y);
      glTexCoord2d(tchar/26.0, 1.0);glVertex2f(x, y);
    }
    x = x + width;

  }

  glEnd();
  glFlush();


}


/*****************************************************************************
 * Function: drawMapgrid
 * Description: Draws the map, based on the global mapgrid variable
 *****************************************************************************/
void drawMapgrid() {

  int32_t i, j;

  // Draw the outer ring
  glBindTexture(GL_TEXTURE_2D, textures[0]);
  for (i = -1; i <= game.grid_height; i++) {
    drawBlock(-1.0, 1.0*i);
    drawBlock(1.0*game.grid_width, 1.0*i);
  }
  for (i = -1; i <= game.grid_width; i++) {
    drawBlock(1.0*i, -1.0);
    drawBlock(1.0*i, game.grid_width);
  }


  // Draw the grid, with blocks, and shaded ground as appropriate
  for (i = 0; i < game.grid_height; i++) {
    for (j = 0; j < game.grid_width; j++) {

      if (game.gridmap[i][j] == 'X') {
	drawBlock(1.0*j, 1.0*i);
      }
      else if ((i == 0) || (game.gridmap[i-1][j] == 'X')) {
	drawGround(1.0*j, 1.0*i, 1);
      }
      else {
	drawGround(1.0*j, 1.0*i, 0);
      }
	

    }
  }

  glEnd();
  glFlush();

}


/*****************************************************************************
 * Function: drawBoom                           
 * Description: Draws an explosion centered at the x,y coordinate position, 
 * of a specified 2D length, using the sprite selected by the option parameter
 *****************************************************************************/
void drawBoom(bomb_type *bomb) {

  float x, y;
  enum BOOM_TYPES option;
  enum BOOM_PARTS part;
  uint8_t radius;
  uint8_t i, j;



  x = bomb->x_pos;
  y = bomb->y_pos;
  radius = bomb->radius;
  option = bomb->boom;

  if (radius == 0)
    return;


  glBegin(GL_QUADS);

  for (i = 0; i < 5; i++) {

    switch (i) {


      case 0: // MID part
	drawBoomPart(x, y, option, MID);
	break;
      case 1: // LEFT part
    	
	for (j = 1; j <= radius; j++) {
	  // Don't explode left if there's a wall there
	  if ((j > (int32_t)x) || 
	      (game.gridmap[(int32_t)y][(int32_t)x-j] == 'X')) {
	    break;
	  }
	  else if (j < radius) {
	    drawBoomPart(x-j, y, option, LPART);
	  }
	  else {
	    drawBoomPart(x-j, y, option, LEND);
	  }
	}

	break;
      case 2: // RIGHT part

	for (j = 1; j <= radius; j++) {
	  // Don't explode right if there's a wall there
	  if (((int32_t)x+j >= game.grid_width) || 
	      (game.gridmap[(int32_t)y][(int32_t)x+j] == 'X')) {
	    break;
	  }
	  else if (j < radius) {
	    drawBoomPart(x+j, y, option, RPART);
	  }
	  else {
	    drawBoomPart(x+j, y, option, REND);
	  }
	}
	break;

      case 3: // UP part
	for (j = 1; j <= radius; j++) {
	  // Don't explode right if there's a wall there
	  if ((j > (int32_t)y) || 
	      (game.gridmap[(int32_t)y-j][(int32_t)x] == 'X')) {
	    break;
	  }
	  else if (j < radius) {
	    drawBoomPart(x, y-j, option, UPART);
	  }
	  else {
	    drawBoomPart(x, y-j, option, UEND);
	  }
	}

	break;
      case 4: // DOWN part

	for (j = 1; j <= radius; j++) {
	  // Don't explode right if there's a wall there
	  if (((int32_t)y+j >= game.grid_height) || 
	      (game.gridmap[(int32_t)y+j][(int32_t)x] == 'X')) {
	    break;
	  }
	  else if (j < radius) {
	    drawBoomPart(x, y+j, option, DPART);
	  }
	  else {
	    drawBoomPart(x, y+j, option, DEND);
	  }
	}

	break;

    }

  }

  glEnd();
}



/*****************************************************************************
 * Function: drawBoomPart                           
 * Description: Draws a part of an explosion, based on the x,y position and 
 * the enumerated bomb part type
 *****************************************************************************/
void drawBoomPart(float x,float y,enum BOOM_TYPES option,enum BOOM_PARTS part) {

  float TC_x1, TC_x2, TC_y1, TC_y2;
 
  switch(part) {
    case MID:
      TC_x1 = 0.1*option;
      TC_x2 = 0.1*option+0.1;
      TC_y1 = 1.0;
      TC_y2 = 0.75;
      break;
    case UPART:
      TC_x1 = 0.1*option;
      TC_x2 = 0.1*option+0.1;
      TC_y1 = 0.25;
      TC_y2 = 0.5;
      break;
    case DPART:
      TC_x1 = 0.1*option;
      TC_x2 = 0.1*option+0.1;
      TC_y1 = 0.5;
      TC_y2 = 0.25;
      break;
    case UEND:
      TC_x1 = 0.1*option;
      TC_x2 = 0.1*option+0.1;
      TC_y1 = 0.5;
      TC_y2 = 0.75;
      break;
    case DEND:
      TC_x1 = 0.1*option;
      TC_x2 = 0.1*option+0.1;
      TC_y1 = 0.75;
      TC_y2 = 0.5;
      break;
    case LPART:
      TC_x1 = 0.2*option+0.1;
      TC_x2 = 0.2*option+0.2;
      TC_y1 = 0.0;
      TC_y2 = 0.25;
      break;
    case RPART:
      TC_x1 = 0.2*option+0.2;
      TC_x2 = 0.2*option+0.1;
      TC_y1 = 0.0;
      TC_y2 = 0.25;      
      break;
    case LEND:
      TC_x1 = 0.2*option;
      TC_x2 = 0.2*option+0.1;
      TC_y1 = 0.0;
      TC_y2 = 0.25;
      break;
    case REND:
      TC_x1 = 0.2*option+0.1;
      TC_x2 = 0.2*option;
      TC_y1 = 0.0;
      TC_y2 = 0.25;
      break;
  }

  glTexCoord2d(TC_x1, TC_y1);glVertex3f(x, y+1.0, -0.25);
  glTexCoord2d(TC_x2, TC_y1);glVertex3f(x+1.0, y+1.0, -0.25);
  glTexCoord2d(TC_x2, TC_y2);glVertex3f(x+1.0, y, -0.25);
  glTexCoord2d(TC_x1, TC_y2);glVertex3f(x, y, -0.25);

}


/*****************************************************************************
 * Function: drawBomb                           
 * Description: Draws a bomb at the x,y coordinate position, using the 
 * sprite selected by the option parameter
 *****************************************************************************/
void drawBomb(bomb_type *bomb) {

  float x, y;
  enum BOMB_TYPES option;

  x = bomb->x_pos;
  y = bomb->y_pos;
  option = bomb->type;

  glBegin(GL_QUADS);

  switch(option) {
    case BOMB1:
      glTexCoord2d(0.0, 0.0);glVertex3f(x, y+1.0, -0.5);
      glTexCoord2d(1.0/3, 0.0);glVertex3f(x+1.0, y+1.0, -0.5);
      glTexCoord2d(1.0/3, 1.0);glVertex3f(x+1.0, y, -0.5);
      glTexCoord2d(0.0, 1.0);glVertex3f(x, y, -0.5);
      break;
    case BOMB2:
    default:
      glTexCoord2d(1.0/3, 0.0);glVertex3f(x, y+1.0, -0.5);
      glTexCoord2d(2.0/3, 0.0);glVertex3f(x+1.0, y+1.0, -0.5);
      glTexCoord2d(2.0/3, 1.0);glVertex3f(x+1.0, y, -0.5);
      glTexCoord2d(1.0/3, 1.0);glVertex3f(x, y, -0.5);
      break;
    case BOMB3:
      glTexCoord2d(2.0/3, 0.0);glVertex3f(x, y+1.0, -0.5);
      glTexCoord2d(1.0, 0.0);glVertex3f(x+1.0, y+1.0, -0.5);
      glTexCoord2d(1.0, 1.0);glVertex3f(x+1.0, y, -0.5);
      glTexCoord2d(2.0/3, 1.0);glVertex3f(x, y, -0.5);
      break;
  }

  glEnd();

}


/*****************************************************************************
 * Function: drawPlayer                           
 * Description: Draws a player at the x,y coordinate position, enumerated
 * by movement status
 *****************************************************************************/
void drawPlayer(player_type *player) {

  float x, y;


  x = player->x_pos;
  y = player->y_pos;


  glBegin(GL_QUADS);

  switch(player->movetype) {
    case LEFT1:
      glTexCoord2d(54.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(70.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(70.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(54.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case LEFT2:
      glTexCoord2d(72.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(89.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(89.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(72.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case LEFT3:
      glTexCoord2d(90.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(108.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(108.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(90.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case RIGHT1:
      glTexCoord2d(70.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(54.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(54.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(70.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case RIGHT2:
      glTexCoord2d(89.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(72.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(72.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(89.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case RIGHT3:
      glTexCoord2d(108.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(90.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(90.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(108.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case UP1:
      glTexCoord2d(167.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(183.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(183.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(167.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case UP2:
      glTexCoord2d(185.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(201.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(201.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(185.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case UP3:
      glTexCoord2d(203.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(219.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(219.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(203.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DOWN1:
      glTexCoord2d(0.0, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(17.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(17.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(0.0, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DOWN2:
      glTexCoord2d(18.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(34.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(34.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(18.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DOWN3:
      glTexCoord2d(35.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(52.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(52.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(35.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DEAD1:
      glTexCoord2d(292.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(308.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(308.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(292.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DEAD2:
      glTexCoord2d(309.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(325.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(325.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(309.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DEAD3:
      glTexCoord2d(326.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(342.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(342.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(326.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case DEAD4:
      glTexCoord2d(343.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(1.0, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(1.0, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(343.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case KICKLEFT:
      glTexCoord2d(222.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(238.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(238.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(222.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case KICKRIGHT:
      glTexCoord2d(239.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(256.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(256.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(239.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case KICKUP:
      glTexCoord2d(275.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(291.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(291.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(275.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;
    case KICKDOWN:
      glTexCoord2d(257.0/358, 0.0);glVertex3f(x, y+1.0, y);
      glTexCoord2d(274.0/358, 0.0);glVertex3f(x+1.0, y+1.0, y);
      glTexCoord2d(274.0/358, 1.0);glVertex3f(x+1.0, y-0.5, y);
      glTexCoord2d(257.0/358, 1.0);glVertex3f(x, y-0.5, y);
      break;

  }


  glEnd();

}


/*****************************************************************************
 * Function: drawBlock                            
 * Description: Draws a non-passable block at the given x,y coordinates. 
 * Note that the glBindTexture should be called first by the application.
 *****************************************************************************/
void drawBlock(float x, float y) {

  glBegin(GL_QUADS);

    glTexCoord2d(0.0, 0.0);glVertex3f(x, y+1.0, -0.75);
    glTexCoord2d(0.25, 0.0);glVertex3f(x+1.0, y+1.0, -0.75);
    glTexCoord2d(0.25, 1.0);glVertex3f(x+1.0, y, -0.75);
    glTexCoord2d(0.0, 1.0);glVertex3f(x, y, -0.75);

  glEnd();

}

/*****************************************************************************
 * Function: drawGround                            
 * Description: Draws a ground tile (shaded if alt = 1).
 * Note that the glBindTexture should be called first by the application.
 *****************************************************************************/
void drawGround(float x, float y, uint8_t alt) {

  glBegin(GL_QUADS);

  if (alt == 1) {
    glTexCoord2d(0.25, 0.0);glVertex3f(x, y+1.0, -1);
    glTexCoord2d(0.50, 0.0);glVertex3f(x+1.0, y+1.0, -1);
    glTexCoord2d(0.50, 1.0);glVertex3f(x+1.0, y, -1);
    glTexCoord2d(0.25, 1.0);glVertex3f(x, y, -1);
  }
  else {
    glTexCoord2d(0.50, 0.0);glVertex3f(x, y+1.0, -1);
    glTexCoord2d(0.75, 0.0);glVertex3f(x+1.0, y+1.0, -1);
    glTexCoord2d(0.75, 1.0);glVertex3f(x+1.0, y, -1);
    glTexCoord2d(0.50, 1.0);glVertex3f(x, y, -1);
  }

  glEnd();

}
 

/*****************************************************************************
 * Function: loadSprites                            
 * Description: Loads the various texture maps used in BoomTown
 *****************************************************************************/
void loadSprites() {

  uint8_t *pixels; 
  uint32_t width, height;

  glGenTextures(NUM_TEXTURES, textures);

  // textures[0] is the background sprites
  pixels = bmpRead("sprites/background.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[0]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[1] is the first team's sprites
  pixels = bmpRead("sprites/teamA.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[1]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[2] is the secondt team's sprites
  pixels = bmpRead("sprites/teamB.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[2]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[3] is the third team's sprites
  pixels = bmpRead("sprites/teamC.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[3]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[4] is the fourth team's sprites
  pixels = bmpRead("sprites/teamD.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[4]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[5] is the bomb sprites
  pixels = bmpRead("sprites/bomb.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[5]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[6] is the explosion sprites
  pixels = bmpRead("sprites/explosion.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[6]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);

  // textures[7] is the scoreboard sprites
  pixels = bmpRead("sprites/scoreboard.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[7]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);


  // textures[8] is the font
  pixels = bmpRead("sprites/font.bmp", &width, &height); 
  glBindTexture(GL_TEXTURE_2D, textures[8]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, 
	       GL_UNSIGNED_BYTE, pixels);
  free(pixels);


}



/*****************************************************************************
 * Function: timerCallback
 * Description: Handles timer events, to keep things at an even FPS
 *****************************************************************************/
static void timerCallback(int value) {


  glutPostRedisplay();
  glutTimerFunc(FRAME_LATENCY, timerCallback, value);

}




/*****************************************************************************
 * Function: keyPress                            
 * Description: configures the game based on keyboard presses
 *****************************************************************************/
void keyPress(uint8_t key, int x, int y) {

  switch(key) {

    case 's':
    case 10:
    case 13:
      if (game.start == 0) {
	game.start = 1;
      }
      else {
	game.start = 0;
      }
      break;
    case 'n':
      if (game.start == 0) {
	updateGame();
      }
      break;
    case 'e':
      game.finish = 1;
      break;
    case 'p':
      printStatus();
      break;
    case 27:
    case 'q':
      exit(0);
      break;
    case 'r':
      compileMap();
      initGame();
      compileTeams();
      game.start = 1;
      break;

  }

}



/*****************************************************************************
 * Function: initGL                            
 * Description: Initializes the OpenGL and GLUT state
 *****************************************************************************/
void initGL(int argc, char **argv) {
  
  // GLUT initialization stuff
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(config.screen_width*(game.grid_width+10), 
		     config.screen_height*(game.grid_height+2));
  glutCreateWindow("BoomTown, v-1.0a");
  
  
  glutDisplayFunc(displayGame);
  glutIgnoreKeyRepeat(1); 
  glutKeyboardFunc(keyPress);
 
  
  glClearColor(0.25, 0.25, 0.25, 1.0);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-5,game.grid_width+5,game.grid_height+1,-1,-1.0*(game.grid_height+1),
	  (game.grid_height+1));

  
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);

  glAlphaFunc(GL_GREATER, 0);
  glEnable(GL_ALPHA_TEST);


  timeold = glutGet(GLUT_ELAPSED_TIME);
  glutTimerFunc(FRAME_LATENCY, timerCallback, 0);


}





